//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// June 02, 1999
//
// IMPORTANT: The flag definitions in this file must match those in
// eventattributes.f90.
//
#ifndef Event_Attributes_h
#define Event_Attributes_h

#include <vector>
#include <string>
#include <Dal.h>

namespace EventAttributes {

  // Attributes can share the same number: it means that they apply to
  // different instruments.

  // Try to keep the info/rejection attributes in the first/last 16
  // bits.

  enum EvAtt {
    DIAGONAL = 0, INVALID_PATTERN = 0,
    CLOSE_TO_CCD_BORDER = 1,
    CLOSE_TO_CCD_WINDOW = 2,
    CLOSE_TO_NODE_BOUNDARY = 3, ON_NODE0 = 3, ON_OFFSET_COLUMN = 3,
    ON_NODE1, NEXT_TO_OFFSET_COLUMN = 4,
    CLOSE_TO_ONBOARD_BADPIX = 5,
    CLOSE_TO_BRIGHTPIX = 6,
    CLOSE_TO_FLICKERINGPIX = 7,
    CLOSE_TO_DEADPIX = 8,
    CLOSE_TO_BADCOL = 9,
    CLOSE_TO_BADROW = 10,
    IN_SPOILED_FRAME = 11,
    UNKNOWN_RAWX_PATTERN = 12, ONBOARD_SER = 12,
    ON_BADOFFSET = 13,    
    FLICKERING = 15,
    OUT_OF_FOV = 16,
    IN_BAD_FRAME = 17,
    OUTSIDE_GTI = 18,
    COSMIC_RAY= 19, BAD_SHAPE = 19,
    MIP_ASSOCIATED = 20,  ON_NODE_INTERFACE = 20,
    ON_BADPIX = 21,
    REJECTED_BY_GATTI, SECONDARY = 22,
    TRAILING, NEXT_TO_CCD_BORDER = 23,
    NEXT_TO_BADPIX = 24,
    OUT_OF_CCD_WINDOW = 25, ON_WINDOW_BORDER = 25,
    OUTSIDE_THRESHOLDS = 26, BELOW_THRESH = 26,
    BELOW_ACCEPTANCE = 27, LOEN_NOISE = 27,
    ON_BADROW = 28,
    BAD_E3E4 = 29,
    UNDERSHOOT = 30,
    
    EVLIST_LOCAL = 31,

    NUMBER_OF
    };

  // These are the rejection masks for the instruments.
  const int RGS_REJECTION_MASK =
    1 << (int) ON_BADPIX |
    1 << (int) NEXT_TO_BADPIX |
    1 << (int) BAD_SHAPE |
    1 << (int) ON_WINDOW_BORDER |
    1 << (int) ON_NODE_INTERFACE |
    1 << (int) BELOW_ACCEPTANCE |
    1 << (int) IN_BAD_FRAME |
    1 << (int) OUTSIDE_GTI;

  const int EMOS_REJECTION_MASK =
    1 << (int) OUT_OF_FOV |
    1 << (int) IN_BAD_FRAME |
    1 << (int) COSMIC_RAY |
    1 << (int) ON_BADPIX |
    1 << (int) REJECTED_BY_GATTI |
    1 << (int) OUT_OF_CCD_WINDOW |
    1 << (int) OUTSIDE_THRESHOLDS |
    1 << (int) ON_BADROW |
    1 << (int) BAD_E3E4 |
    1 << (int) UNDERSHOOT |
    1 << (int) FLICKERING |
	1 << (int) ON_BADOFFSET;
  
  const int EPN_REJECTION_MASK =
    1 << (int) IN_BAD_FRAME |
    1 << (int) COSMIC_RAY |
    1 << (int) ON_BADPIX |
    1 << (int) SECONDARY |
    1 << (int) TRAILING |
    1 << (int) MIP_ASSOCIATED |
    1 << (int) BELOW_THRESH |
    1 << (int) LOEN_NOISE ;
	
  // SCR 198	
  const int	EMOS_SPECTRAL_MASK =
    ~(1 << (int) CLOSE_TO_DEADPIX);
  
  const int CLEAR_ALL_FLAGS = 0;
  const string COLUMN_NAME = "FLAG";
  const Dal::DataType COLUMN_TYPE = Dal::Int32;

  // Set the bit corresponding to the event attribute
  void setEventAttribute(int & v, const EvAtt n);

  // Add a table attribute
  void setEventAttribute(Table *table, const EvAtt n);

  // Propagate table attributes from one table to another
  void copyEventAttributes(const Table *fromtable, Table *totable);

  // Specialized functions to set instrument specific rejection masks
  // with a fixed name
  void setEmosRejectionMask(Table *table, int mask = EMOS_REJECTION_MASK);
  void setEpnRejectionMask(Table *table, int mask = EPN_REJECTION_MASK);
  void setRgsRejectionMask(Table *table, int mask = RGS_REJECTION_MASK);
  
  // SCR 198
  void setEmosSpectralMask(Table *table, int mask = EMOS_SPECTRAL_MASK);

  const vector<string> names();
  EvAtt lookup(const string & name);
  string nameOfEventAttribute(const Table *table, const EvAtt n);

  bool eventAttributeIsSet(const Table *table, const EvAtt n);
  bool eventAttributeIsSet(int & v, const EvAtt n);

  // Hook to write event attributes as a preference file (see sasprefgen)
  void EventAttributes::writePreferenceFile(ostream & os);

  // Overloaded functions:
  //  - setting an event attribute the selection expression needs !=
  //  - setting the mask the selection expression needs ==
  void writeSelectExpression(ostream & os, EvAtt n);
  void writeSelectExpression(ostream & os, int mask);


};

#endif
