! Module: addtable.f90 
! Author: P.Videler
! Version: 0.1
! 
! (c) 1998 European Space Agency 
!
! Description: Add table to existing table.
!! 
! Comments:  -  All columns of existing table must be present in new table.
!            -  Attributes existing table remain unchanged.
! 
! Modifications: 
! 
! $Id: addtables.f90,v 1.3 2000/02/04 22:31:06 gvacanti Exp $

module addtables
  use dal         
  use errorHandling

  implicit none 

  !......................................................................
  private    ! change default to private
  public  ::  addRows
  !......................................................................

  interface addRows
     module procedure addTable2Table
  end interface

contains 

  ! ====================================================================
  subroutine addTable2Table (tbl1, tbl2) 
    ! ====================================================================
    ! add corresponding columns from table 2 to table 1

    type (tableT),    intent(inout)   :: tbl1  
    type (tableT),    intent(in)      :: tbl2  

    type (ColumnT)                    :: col1, col2
    integer                           :: i, oldsize
    character(len=*),    parameter    :: fn='addtables: '

    ! Check if all columns of table1 are availeble in table2.
    ! This check is not really necessary but done here to produce a 'nice'
    ! error message before DAL gives access errors for non-existing columns. 
    ! Matching of data types is not checked. You'll get a Dal error.

    do i = 0, numberOfColumns(tbl1) - 1 
       col1 = column(tbl1, i, modify)
       if (.not. hasColumn(tbl2, name(col1))) then
          write(errstr,*) ' Can not combine tables. ', &
               & 'Table '//trim(name(tbl1))//' has column ', & 
               & trim(name(col1))//' which is not found in table '//trim(name(tbl2))//' .'
          call error (fn, trim(errstr)) 
       endif
       call release(col1)
    enddo

    ! expand size table1

    oldsize = numberOfRows(tbl1)
    call insertRows(tbl1, oldsize, numberOfRows(tbl2))

    ! loop over columns table1

    do i = 0, numberOfColumns(tbl1) - 1 

       col1 = column(tbl1, i, modify)
       col2 = column(tbl2, name(col1), modify)

       ! datatyes names from dal.f90

       select case (dataType(col1))     
       case(integer8)  ; call combineColumnInt8  (col1, col2, oldsize)  
       case(integer16) ; call combineColumnInt16 (col1, col2, oldsize)  
       case(integer32) ; call combineColumnInt32 (col1, col2, oldsize)  
       case(real32)    ; call combineColumnSingle(col1, col2, oldsize)  
       case(real64)    ; call combineColumnDouble(col1, col2, oldsize)  
       case default    ; call error (fn//'datatype not implemented')
       end select

    enddo

  end subroutine

  ! ------------------------------------------------
  subroutine combineColumnInt8 (col1, col2, fromrow)
    !------------------------------------------------
    type (columnT) , intent(inout)   :: col1, col2
    integer        , intent(in)      :: fromrow
    integer (int8), pointer          :: p(:)

    p => int8Data(col1)
    p(fromrow:) = int8Data(col2)

  end subroutine

  ! ------------------------------------------------
  subroutine combineColumnInt16 (col1, col2, fromrow)
    !------------------------------------------------
    type (columnT) , intent(inout)   :: col1, col2
    integer        , intent(in)      :: fromrow
    integer (int16), pointer         :: p(:)

    p => int16Data(col1)
    p(fromrow:) = int16Data(col2)

  end subroutine

  !-------------------------------------------------
  subroutine combineColumnInt32 (col1, col2, fromrow)
    !------------------------------------------------
    type (columnT) , intent(inout)   :: col1, col2
    integer        , intent(in)      :: fromrow
    integer (int32), pointer         :: p(:)

    p => int32Data(col1)
    p(fromrow:) = int32Data(col2)

  end subroutine

  !-------------------------------------------------
  subroutine combineColumnSingle (col1, col2, fromrow)
    !------------------------------------------------
    type (columnT) , intent(inout)   :: col1, col2
    integer        , intent(in)      :: fromrow
    real (kind=single), pointer      :: p(:)

    p => real32Data(col1)
    p(fromrow:) = real32Data(col2)

  end subroutine 

  !-------------------------------------------------
  subroutine combineColumnDouble (col1, col2, fromrow)
    !------------------------------------------------
    type (columnT) , intent(inout)   :: col1, col2
    integer        , intent(in)      :: fromrow
    real (kind=double), pointer      :: p(:)

    p => real64Data(col1)
    p(fromrow:) = real64Data(col2)

  end subroutine 

end module
















