! Module: generalStuff.f90 
! Author: P.Videler
! Version: 1.3
! 
! (c) 1997 European Space Agency 
!
! Description: various general routines
! 
! Comments: 
! 
! Modifications: 
! 
! $Id: generalstuff.f90,v 1.5 2000/02/17 14:32:55 pvideler Exp $
! 
module generalStuff

 use types         ! SAS data types
 use errorHandling

 implicit none 

 !......................................................................
 private    ! change default to private
 public  ::         &
      Str,          &        
      cumulatedSum, &
      compareSize,  &
      monotonicIncreasing, &
      CorrectWrapAround
 !......................................................................

 ! convert to string
 interface Str
    module procedure double2string, single2string, integer2string
 end interface


 interface compareSize
    module procedure compare1_SingleSingle, compare1_SingleInt
    module procedure compare2_SingleSingle, compare2_SingleInt
 end interface

 interface monotonicIncreasing
    module procedure increasing_int, increasing_real
 end interface


contains 

 ! ====================================================================
 ! convert real and integer of various types to string of length=30
 ! Do you know how to return variable length string??. mailto:PeterV@writeme.com)
 !-----------------------------------------------------------
 Function double2string(x) result(string)
  character(len=30)  :: string
  real(double)           :: x
  write(string,*) x
 end Function 

 Function single2string(x) result(string)
  character(len=30)  :: string
  real(single)           :: x
  write(string,*) x
 end Function 

 Function integer2string(i) result(string)
  character(len=30)  :: string
  integer            :: i
  write(string,*) i
 end Function 

 ! ====================================================================
 function cumulatedSum (xx) result(rr) 
  ! ====================================================================
  ! normalized running sum of array: rr(i) = SUM(xx(1:i))/SUM(xx) 
  ! note: IEEE exception if SUM(xx)=0

  real(single), intent(in) :: xx(:)
  real(single)             :: rr(size(xx)) ! note: array-valued function 
  !                                        !  see eg MetCalf & Reid pg.127
  real(single)   :: total
  integer        :: i

  !  This works but .....
!!$  do i=1, size(xx)
!!$     rr(i) = sum(xx(1:i))
!!$  end do
!!$  rr = rr/sum(rr)

  ! ..... faster doing it yourself.
  total = 0
  do i=1, size(xx)
     total = total + xx(i)
     rr(i) = total
  end do
  rr = rr/total

 end function cumulatedSum


 ! ====================================================================

 !---------------------------------------------------------------------------
 subroutine compare1_SingleSingle(v1, v2, fn)
  !---------------------------------------------------------------------------
  real(single),     intent(in) :: v1(:), v2(:)
  character(len=*), intent(in) :: fn

  if (size(v1) /= size(v2))  then 
     write(errstr,*) fn//': vector sizes not equal. Size1: ',size(v1),' :Size2 ',size(v2)
     call error ('histo')
  endif

 end subroutine compare1_SingleSingle

 !---------------------------------------------------------------------------
 subroutine compare2_SingleSingle(xx1, xx2, fn)
  !---------------------------------------------------------------------------
  real(single),  intent(in)    :: xx1(:,:), xx2(:,:)
  character(len=*), intent(in) :: fn

  if (ANY(shape(xx1) /= shape(xx2)))  then 
     write(errstr,*) fn//': array shapes not equal. Shape1: ',shape(xx1),' :Shape2 ',shape(xx2)
     call error ('histo')
  endif

 end subroutine compare2_SingleSingle

 !---------------------------------------------------------------------------
 subroutine compare1_SingleInt(v1, v2, fn)
  !---------------------------------------------------------------------------
  real(single),     intent(in) :: v1(:)
  integer,          intent(in) :: v2(:)
  character(len=*), intent(in) :: fn

  if (size(v1) /= size(v2))  then 
     write(errstr,*) fn//': vector sizes not equal. Size1: ',size(v1),' :Size2 ',size(v2)
     call error ('histo')
  endif

 end subroutine compare1_SingleInt

 !---------------------------------------------------------------------------
 subroutine compare2_SingleInt(xx1, xx2, fn)
  !---------------------------------------------------------------------------
  real(single),  intent(in)    :: xx1(:,:)
  integer,       intent(in)    :: xx2(:,:)
  character(len=*), intent(in) :: fn

  if (ANY(shape(xx1) /= shape(xx2)))  then 
     write(errstr,*) fn//': array shapes not equal. Shape1: ',shape(xx1),' :Shape2 ',shape(xx2)
     call error ('histo')
  endif

 end subroutine compare2_SingleInt

 ! ====================================================================


  !---------------------------------------------------------------------------
 function increasing_int (xx) result(increasing) 
  !---------------------------------------------------------------------------

  integer,   intent(in) :: xx(:)
  logical               :: increasing
  integer               :: i

  increasing=.true.
  do i=1, size(xx)-1
     if (xx(i+1) < xx(i)) then
        increasing = .false.
        ! write(*,*) 'increasing_int  =', i, xx(i), xx(i+1)
        EXIT
      endif
  enddo

 end function increasing_int

  !---------------------------------------------------------------------------
 function increasing_real (xx) result(increasing) 
  !---------------------------------------------------------------------------

  real,      intent(in) :: xx(:)
  logical               :: increasing
  integer               :: i

  increasing=.true.
  do i=1, size(xx)-1
     if (xx(i+1) < xx(i)) then
        increasing = .false.
        EXIT
      endif
  enddo

 end function increasing_real

 ! --------------------------------------------------------------------------------
  subroutine CorrectWrapAround(array, nflips)
    ! --------------------------------------------------------------------------------

    ! fix wrap around int16 problem. (Peter Videler 20jan99)

    ! This patch corrects an int16 column for 2 effects:
    !  - sign change from 2^15 to -2^15  (reading unsigned as signed)
    !               and/or 
    !  - change from 2^16 -1 to 0  (unsigned 16 bit overflow)

    ! note: It is assumed that a change of ANY two adjacent array values by 
    !       more than 2^15 is due to the bit overflow!!!

    !  Simple, straight forward approach.
    !  Can probably be made more efficient using fancy bit manipulations routines.

    integer(kind=int32), dimension(:), intent(inout) :: array
    integer,                           intent(out)   :: nflips

    integer,  parameter  :: tothe16th = 65536
    integer,  parameter  :: tothe15th = 32768
    integer              :: i, current, previous, offset

    nflips = 0
    offset = 0 

    if (size(array) < 2) RETURN

    previous = array(1)           ! array index start at 1

    do i=2, size(array)
       current = array(i)
       if ((previous - current) > tothe15th) offset = offset + tothe16th 
       previous = current 
       if (offset > 0) array(i) = current + offset
    enddo

    nflips = offset/tothe16th
    if (nflips > 0) then
       write(errstr,*) 'Array values corrected for int16 wraparound. ', nflips, ' sign flips'
       call warning('CorrectWrapAround')
    endif

  end subroutine CorrectWrapAround

end module generalStuff







